package servidor;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;


public class Servidor implements Runnable {

    private ServerSocket server;
    
    final private List<Atencion> atendidos;
    
    private boolean inicializado;
    private boolean ejecutando;
    
    private Thread thread;
    
    public Servidor (int puerto) throws Exception {
        atendidos = new ArrayList<Atencion>();
        
        inicializado = false;
        ejecutando = false;
        
        open(puerto);
    }
    
    private void open(int puerto) throws Exception {
        server = new ServerSocket(puerto);
        inicializado = true;
        
    }
   
    private void close () {
        for (Atencion atendido : atendidos){
            try {
                atendido.stop();
            }
            catch(Exception e){
                System.out.println(e);
            }
        }
        
        try{
            server.close();
        }
        catch(Exception e){
            System.out.println(e);
        }
        
        server = null;
        
        inicializado = false;
        ejecutando = false;
        
        thread = null;
        
    }
    
    
    public void start () {
        if(!inicializado || ejecutando){
            return;
        }
        
        ejecutando = true;
        thread = new Thread(this);
        thread.start();        
    }
    
    public void stop() throws Exception {
        ejecutando = false;
        
       if (thread != null){
        thread.join();
       }
    }
    
    public void run() {
        System.out.println("Aguardando conexion.");
        
        while (ejecutando) {
            try {
                server.setSoTimeout(2500);
                
                Socket socket = server.accept();
                
                System.out.println("Conexion Establecida.");
                Atencion atencion = new Atencion(socket);
                atencion.start();
                
                atendidos.add(atencion);
                        
            }
            catch (SocketTimeoutException e){
                // ignorar
            }
            catch (Exception e){
                System.out.println("ACÁ?"+e);
                break;
            }
        }
        
        close();
        
    }
   
    public static void main(String[] args) throws Exception {

        System.out.println("Iniciando servidor.");
        
        Servidor servidor = new Servidor(2525);    
        servidor.start();

        System.out.println("Presione ENTER para cerrar conexión.");
        new Scanner(System.in).nextLine();
    

        System.out.println("Cerrando Servidor.");
        servidor.stop();
        
        
    }
    
}
